#include <stdio.h>
#include <stdlib.h>
#include <wiringPi.h>
#include <wiringPiSPI.h>

#define AQM1248Channel  0
#define AQM1248RS	4

//   LCD表示器 AQM1248A 制御関数
//   LCDWriteRS0(int channel, int data)
//   LCDWriteRS1(int channel, int data)
LCDWriteRS0(unsigned char data) {
    digitalWrite(AQM1248RS, 0);
    wiringPiSPIDataRW(AQM1248Channel, &data, 1);
}

LCDWriteRS1(unsigned char  data) {
    digitalWrite(AQM1248RS, 1);
    wiringPiSPIDataRW(AQM1248Channel, &data, 1);
}

//   LCD表示器 AQM1248A 初期化
//   AQM1248Init(int fd)
AQM1248Init() {
    delay(40);
    LCDWriteRS0(0xae);			// Display = off
    LCDWriteRS0(0xa0); 			// ADC = normal
    LCDWriteRS0(0xc8); 			// Common out = reverse
    LCDWriteRS0(0xa3); 			// bias = 1/7

    LCDWriteRS0(0x2c); delay(2);	// power control 1
    LCDWriteRS0(0x2e); delay(2);	// power control 2
    LCDWriteRS0(0x2f); delay(2);	// power control 3

    LCDWriteRS0(0x23); 			// Vo voltage resistor retio set
    LCDWriteRS0(0x81); 			// Electronic volume mode set
    LCDWriteRS0(0x1c); 			// Electronic volume value set

    LCDWriteRS0(0xa4); 			// display all point = normal
    LCDWriteRS0(0x40); 			// display start line = 0
    LCDWriteRS0(0xa6); 			// display normal / reverse = normal
    LCDWriteRS0(0xaf); 			// display = ON
}

//   8ビットの0-7ビットを反転する
//   int revbit8(int v)
//   参考 http://graphics.stanford.edu/~seander/bithacks.html#ReverseParallel
int revbit8(int v) {
    v = ((v >> 1) & 0x55) | ((v & 0x55) << 1);	// 偶数奇数ビットをスワップ
    v = ((v >> 2) & 0x33) | ((v & 0x33) << 2);	// 2ビットずつスワップ
    v = ((v >> 4) & 0x0f) | ((v & 0x0f) << 4);	// 4ビットずつスワップ
    return(v);
}

//   bmpファイル読み込み
//   bmpin(int *buf)
bmpin(int buf[][6]) {
    int i;
    FILE *fd;
    int page;
    int column;

    // ---- bmpファイルからの読み取り ----
    if ((fd = fopen("List5-1.bmp","rb")) == NULL) {
        fprintf(stderr, "List5-1.bmpのオープンに失敗しました。\n");
        exit(-1);
    }
    for (i=0; i<0x3e; i++) fgetc(fd);   // ヘッダ読み飛ばし
    for (column=0; column<128; column++) {
        for (page=0; page<6; page++) {
                buf[column][page] = revbit8(fgetc(fd));
        }
        fgetc(fd);                      // 読み飛ばし
        fgetc(fd);                      // 読み飛ばし
    }
}

//　メイン
main() {
    int buf[128][6];
    int page, column;

    wiringPiSetup();
    wiringPiSPISetupMode(0, 500000,3);	// SPI 50kBPS Mode3
    pinMode(AQM1248RS, OUTPUT);		// AQM1248 RS端子
    AQM1248Init();			// AWM1248初期化

    bmpin(buf);				// データ読み込み

    // ---- データ出力 ----
    for(page=0; page < 6; page++){
	LCDWriteRS0(0xb0 | page);
	LCDWriteRS0(0x10);
	LCDWriteRS0(0x00);
	for(column=0; column < 128; column++) {
	    LCDWriteRS1(buf[column][page]);
	}
    }
}

